<?php

declare(strict_types = 1);

namespace Drupal\entity_editor_tabs;

use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\entity_editor_tabs\Plugin\Menu\LocalTask\EetCanonicalLocalTask;
use Drupal\entity_editor_tabs\Plugin\Menu\LocalTask\EetLatestLocalTask;
use Drupal\entity_editor_tabs\Plugin\Menu\LocalTask\EetUpdateLocalTask;
use Drupal\entity_route_context\EntityRouteContextRouteHelperInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Hooks for Entity Editor Tabs.
 */
class EetHooks implements ContainerInjectionInterface {

  /**
   * Provides discovery and retrieval of entity type bundles.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected EntityTypeBundleInfoInterface $bundleInfo;

  /**
   * Manages entity type plugin definitions.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Utility for Entity Editor Tabs.
   *
   * @var \Drupal\entity_editor_tabs\EetUtility
   */
  protected EetUtility $eetUtility;

  /**
   * Route helper.
   *
   * @var \Drupal\entity_route_context\EntityRouteContextRouteHelperInterface
   */
  protected EntityRouteContextRouteHelperInterface $routeHelper;

  /**
   * General service for moderation-related questions about Entity API.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface|null
   */
  protected ?ModerationInformationInterface $moderationInformation;

  /**
   * Class that manages modules in a Drupal installation.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();
    $instance->bundleInfo = $container->get('entity_type.bundle.info');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->eetUtility = $container->get('entity_editor_tabs.utility');
    $instance->routeHelper = $container->get('entity_route_context.route_helper');
    $instance->moderationInformation = $container->get('content_moderation.moderation_information', ContainerInterface::NULL_ON_INVALID_REFERENCE);
    $instance->moduleHandler = $container->get('module_handler');
    return $instance;
  }

  /**
   * Implements hook_local_tasks_alter().
   *
   * @see \entity_editor_tabs_local_tasks_alter()
   */
  public function hookLocalTasksAlter(array &$localTasks): void {
    foreach ($this->bundles() as [$entityTypeId, $bundle]) {
      if (!$this->moduleHandler->moduleExists('layout_builder') && !$this->moduleHandler->moduleExists('content_moderation')) {
        continue;
      }

      $canonicalLocalTaskId = $this->getLocalTaskIdFromLinkTemplate($entityTypeId, 'canonical', $localTasks);
      $editFormLocalTaskId = $this->getLocalTaskIdFromLinkTemplate($entityTypeId, 'edit-form', $localTasks);
      $latestVersionLocalTaskId = $this->getLocalTaskIdFromLinkTemplate($entityTypeId, 'latest-version', $localTasks);
      $deleteFormLocalTaskId = $this->getLocalTaskIdFromLinkTemplate($entityTypeId, 'delete-form', $localTasks);
      // From \Drupal\layout_builder\Plugin\SectionStorage\OverridesSectionStorage::buildLocalTasks:
      $overrideTaskId = sprintf('layout_builder_ui:layout_builder.overrides.%s.view', $entityTypeId);
      $overrideTaskId = isset($localTasks[$overrideTaskId]) ? $overrideTaskId : NULL;

      // Re-order tasks so view and edit tasks are respectively together.
      // This strategy attempts to keep the original weight of each task if
      // they are already in the correct order.
      // Tasks and their expected order.
      $tasks = [
        $canonicalLocalTaskId,
        $latestVersionLocalTaskId,
        $editFormLocalTaskId,
        $overrideTaskId,
        $deleteFormLocalTaskId,
      ];
      // Remove non existent:
      $tasks = array_filter($tasks);

      $lastWeight = (int) ($localTasks[$canonicalLocalTaskId]['weight'] ?? NULL);
      // Never modify canonical weight.
      array_shift($tasks);
      foreach ($tasks as $taskId) {
        $taskWeight = (int) ($localTasks[$taskId]['weight'] ?? NULL);
        if ($taskWeight <= $lastWeight) {
          $localTasks[$taskId]['weight'] = $lastWeight + 1;
        }
        $lastWeight = $localTasks[$taskId]['weight'];
      }

      if ($this->moduleHandler->moduleExists('layout_builder') && $this->eetUtility->isLayoutBuilderOverridable($entityTypeId, $bundle)) {
        // 'Edit' tab override:
        if ($editFormLocalTaskId) {
          $localTasks[$editFormLocalTaskId]['class'] = EetUpdateLocalTask::class;
        }

        // 'Layout' tab override:
        if ($overrideTaskId) {
          $localTasks[$overrideTaskId]['title'] = \t('Edit content');
        }
      }

      if ($this->moduleHandler->moduleExists('content_moderation') && $this->moderationInformation->shouldModerateEntitiesOfBundle($this->entityTypeManager->getDefinition($entityTypeId), $bundle)) {
        // 'View' tab override:
        if ($canonicalLocalTaskId) {
          $localTasks[$canonicalLocalTaskId]['class'] = EetCanonicalLocalTask::class;
        }

        // 'Latest version' tab override:
        if ($latestVersionLocalTaskId) {
          $localTasks[$latestVersionLocalTaskId]['class'] = EetLatestLocalTask::class;
        }
      }
    }
  }

  /**
   * Generates all bundles in the system as a flat series of arrays.
   */
  protected function bundles() {
    foreach ($this->bundleInfo->getAllBundleInfo() as $entityTypeId => $info) {
      foreach (array_keys($info) as $bundle) {
        yield [$entityTypeId, $bundle];
      }
    }
  }

  /**
   * Finds the first local task representing a link template.
   *
   * @param string $entityTypeId
   *   The entity type ID.
   * @param string $linkTemplate
   *   The link template.
   * @param array $localTasks
   *   Local task definitions.
   *
   * @return string|null
   *   The local task ID in $localTasks, or NULL if no task was found.
   */
  protected function getLocalTaskIdFromLinkTemplate(string $entityTypeId, string $linkTemplate, array $localTasks): ?string {
    $routes = $this->routeHelper->getRouteNames($entityTypeId);
    $routeName = $routes[$linkTemplate] ?? NULL;
    foreach ($localTasks as $id => $definition) {
      if (($definition['route_name'] ?? NULL) === $routeName) {
        return $id;
      }
    }
    return NULL;
  }

}
