<?php

declare(strict_types = 1);

namespace Drupal\entity_editor_tabs\Plugin\Menu\LocalTask;

use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Menu\LocalTaskDefault;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides a local task with customization when content moderation is enabled.
 */
class EetLatestLocalTask extends LocalTaskDefault implements ContainerFactoryPluginInterface {

  use StringTranslationTrait;
  use EetLocalTaskTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Moderation information.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected ModerationInformationInterface $moderationInformation;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->stringTranslation = $container->get('string_translation');
    $instance->routeMatch = $container->get('current_route_match');
    $instance->contextRepository = $container->get('context.repository');
    $instance->entityRouteHelper = $container->get('entity_route_context.route_helper');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->moderationInformation = $container->get('content_moderation.moderation_information');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getTitle(Request $request = NULL) {
    $entity = $this->getEntity();
    if (!$entity) {
      return parent::getTitle($request);
    }

    if (!$this->moderationInformation->shouldModerateEntitiesOfBundle($this->entityTypeManager->getDefinition($entity->getEntityTypeId()), $entity->bundle())) {
      return parent::getTitle($request);
    }

    if ($this->moderationInformation->isLiveRevision($entity)) {
      // No customization if there is unpublished forward revision.
      // @todo make this configurable.
      return parent::getTitle($request);
    }

    /** @var \Drupal\Core\Entity\RevisionableStorageInterface $storage */
    $storage = $this->entityTypeManager->getStorage($entity->getEntityTypeId());
    $latestRevisionId = $storage->getLatestRevisionId($entity->id());
    if (!$latestRevisionId) {
      return parent::getTitle($request);
    }

    $latestRevision = $storage->loadRevision($latestRevisionId);
    $state = $this->moderationInformation->getOriginalState($latestRevision);

    return $this->t('View Latest @state', [
      '@state' => $state->label(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags(): array {
    $entity = $this->getEntity();
    return array_merge(
      parent::getCacheTags(),
      ($entity ? $entity->getCacheTags() : []),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts(): array {
    $contexts = parent::getCacheContexts();
    $contexts[] = 'route';
    return $contexts;
  }

}
